package org.netkernel.layer0.bnf;

import java.io.IOException;
import java.io.StringWriter;

import org.netkernel.container.IMessages;
import org.netkernel.layer0.meta.GrammarException;
import org.netkernel.layer0.util.OrderedCheapMultiStringMap;
import org.netkernel.layer0.util.XMLUtils;


public class RegularGroup extends GroupImpl
{
	private final int mMin;
	private final int mMax;
	private NamedGroups mInnerGroups;
	
	public RegularGroup(IGrammarGroup aParent, String aName, int aMin, int aMax, IPartEncoding aEncoding)
	{	super(aParent,aName,aEncoding);
		mMax=aMax;
		mMin=aMin;
	}
	
	public int getMax()
	{	return mMax;
	}

	public int getMin()
	{	return mMin;
	}	
	
	public void writeXML(StringWriter aSW) throws IOException
	{	
		aSW.write("<group");
		
		String groupName=getName();
		int min=getMin();
		int max=getMax();
		
		if (groupName!=null || min!=1 || max>1)
		{	aSW.write(" ");
		}
		if (groupName!=null)
		{	XMLUtils.writeAttribute(aSW,"name",groupName);
		}
		if (min!=1)
		{	XMLUtils.writeAttribute(aSW,"min",Integer.toString(min));
		}
		if (max>1)
		{	String maxString;
			if (max==Integer.MAX_VALUE)
			{	maxString="*";
			}
			else
			{	maxString=Integer.toString(max);
			}
			XMLUtils.writeAttribute(aSW,"max",maxString);
		}
		
		
		aSW.write(">");
		for (int i=0; i<getPartCount(); i++)
		{	getPart(i).writeXML(aSW);
		}
		aSW.write("</group>");
	}
	
	protected void buildRegex(StringWriter aSW)
	{
		int min=getMin();
		int max=getMax();
		
		boolean omitQuantifier = getParent() instanceof Interleave;
		
		if (max==1)
		{	
			if (!omitQuantifier)
			{	aSW.write('(');
				mGroups.putGroup(this);
			}
			int count=getPartCount();
			for (int i=0; i<count; i++)
			{	getPart(i).writeRegex(aSW, mGroups);
			}
			if (!omitQuantifier)
			{	aSW.write(')');
				appendQuantifier(aSW,min,max);
			}
		}
		else
		{
			StringWriter sw = new StringWriter(128);
			mInnerGroups = new NamedGroups();
			
			if (!omitQuantifier)
			{	sw.write('(');
			}
			int count=getPartCount();
			for (int i=0; i<count; i++)
			{	getPart(i).writeRegex(sw, mInnerGroups);
			}
			if (!omitQuantifier)
			{	sw.write(')');
			}
			String innerRegexString=sw.toString();

			
			if (!omitQuantifier)
			{	mGroups.putGroup(this);
				mGroups.skipGroup();
				aSW.write("(");
				mGroups.skipGroups(mInnerGroups.size());
				aSW.write(innerRegexString);
				appendQuantifier(aSW,min,max);
				aSW.write(')');
				setInnerRegex(innerRegexString);
			}
			else
			{	mGroups=mInnerGroups;
				aSW.write(innerRegexString);
				mInnerGroups=null;
			}
					
		}
	}
		
	public void parse(String[] aCapturedGroups, OrderedCheapMultiStringMap aResults, int aOffset)
	{
		String name=getName();
		String capturedWhole=aCapturedGroups[aOffset];
		
		
		boolean omitQuantifier = getParent() instanceof Interleave;
		
		if (omitQuantifier)
		{	//inside interleave
			if (name!=null && capturedWhole!=null)
			{	aResults.put(name, getEncoding().decode(capturedWhole));
			}
			for (int i=0; i<mPartsArray.length; i++)
			{	IGrammarPart p = mPartsArray[i];
				if (p instanceof IGrammarGroup)
				{	IGrammarGroup g=(IGrammarGroup)p;
					g.parse(aCapturedGroups, aResults, aOffset+g.getCapturedGroupIndexInParent()+1);
				}
			}
		}
		else 	if (!hasInnerRegex())
		{	// simple case - there isn't potential for >1
			if (name!=null && capturedWhole!=null)
			{	aResults.put(name, getEncoding().decode(capturedWhole));
			}
			
			for (int i=0; i<mPartsArray.length; i++)
			{	IGrammarPart p = mPartsArray[i];
				if (p instanceof IGrammarGroup)
				{	IGrammarGroup g=(IGrammarGroup)p;
					g.parse(aCapturedGroups, aResults, aOffset+g.getCapturedGroupIndexInParent());
				}
			}
		}
		else
		{	// complex case - regex can't extract each match when >1 so we need to a sub-match and iterate
			String capturedGroup = aCapturedGroups[aOffset];
			if (capturedGroup!=null)
			{
				String[][] matches = findMatchingGroups(capturedGroup);
				for (int j=0; j<matches.length; j++)
				{	String[] cgs = matches[j];
					if (name!=null)
					{	aResults.put(name, getEncoding().decode(cgs[0]));
					}
					for (int i=0; i<mInnerGroups.size(); i++)
					{	IGrammarGroup g=mInnerGroups.getGroup(i);
						if (g!=null)
						{	g.parse(cgs, aResults, i+1);
						}
					}
				}
			}			
		}		
	}

	public String constructIdentifier(OrderedCheapMultiStringMap aPartsRemaining, IMessages aMessages) throws GrammarException
	{
		if (getMax()==1)
		{	try
			{	int s=aPartsRemaining.size();
				String r= innerConstructIdentifier(aPartsRemaining,aMessages);
				if (getMin()==0 && s==aPartsRemaining.size())
				{	//optional and nothing used
					return "";
				}
				else
				{	return r;
				}
			}
			catch (GrammarException e)
			{	if (getMin()!=0)
				{	throw e;
				}
				else
				{	return "";
				}
			}
		}
		else
		{	StringWriter sw = new StringWriter(128);
			int c=0;
			try
			{	while (c<getMax())
				{	String subGroup = innerConstructIdentifier(aPartsRemaining,aMessages);
					sw.write(subGroup);
					c++;
				}
			}
			catch (GrammarException e)
			{	if (c<getMin())
				{	throw e;
				}
			}
			return sw.toString();
		}
	}
	
	public String innerConstructIdentifier(OrderedCheapMultiStringMap aPartsRemaining, IMessages aMessages) throws GrammarException
	{	
		String name=getName();
		if (name!=null)
		{	String result=(String)aPartsRemaining.remove(name);
			if (result!=null)
			{	return getEncoding().encode(result);
			}
		}
		
		int count=getPartCount();
		StringWriter sw = new StringWriter(128);
		for (int i=0; i<count; i++)
		{	Object part = getPart(i);
			if (part instanceof StaticPart)
			{	sw.write( ((StaticPart)part).getContent() );
			}
			else if (part instanceof RegexPart)
			{	name=GrammarImpl.locateClosestName(this);
				throw new GrammarException(aMessages.format("MSG_BNF_MISSING", name));
			}
			else if (part instanceof IGrammarGroup)
			{	String subGroup=((IGrammarGroup)part).constructIdentifier(aPartsRemaining,aMessages);
				sw.write(subGroup);
			}
				
		}
		String result = sw.toString();
		return result;
	}	
}