/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf;

import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ISpace;

/**
 * A request created within a physical endpoint.
 *
 * @author tab
 */
public interface INKFRequest
{   
	/** Header to declare that this sub-request may appear unresponsive for a number of milliseconds and
	 * should not be terminated by any system level tool that deems it deadlocked. Header value should be
	 * a Long value.
	 */
	public static final String HEADER_LIVENESS="liveness";
	
	/** Header to declare that this request should not have its dependencies tracked because it is likely
	 * to have too many and that may cause the system to run out of memory tracking them. By not tracking
	 * dependencies a response will always be expired and will have an unexpectedly low cost.
	 * Existence of header causes dependencies to be forgot.
	 */
	public static final String HEADER_FORGET_DEPENDENCIES=IRequest.HEADER_FORGET_DEPENDENCIES;
	
	/** Header to declare that this request should not have it's response cached and that it's response
	 * should not be retrieved from cache. Usually it is for the endpoint providing
	 * the response to determine if the representation should be cached. In some rare situations it is useful for the client
	 * to force no caching. 
	 * Existence of header inhibits caching.
	 */
	public static final String HEADER_NO_CACHE=IRequest.HEADER_NO_CACHE;
	
	/** Sets the verb of the request encoded as an integer value.
	 * If this method is not called the request verb defaults to SOURCE.
	 *
	 * @see INKFRequestReadOnly#VERB_SOURCE
	 * @see INKFRequestReadOnly#VERB_SINK
	 * @see INKFRequestReadOnly#VERB_NEW
	 * @see INKFRequestReadOnly#VERB_EXISTS
	 * @see INKFRequestReadOnly#VERB_DELETE
	 * @see INKFRequestReadOnly#VERB_META
	 * @see INKFRequestReadOnly#VERB_TRANSREPRESENT
	 * @see INKFRequestReadOnly#VERB_FRAGMENT
	 * @see INKFRequestReadOnly#VERB_META
	 * @see INKFRequestReadOnly#VERB_RESOLVE
	 *
	 * @param aVerb the verb for the request encoded as an integer.
	 */
	void setVerb(int aVerb);
	
	/** Indicate the required type (class or interface) of the object returned as
	 * the resource representation when this request is processed.
	 * If this method is not called, the requested representation defaults to
	 * java.lang.Object, which indicates any type may be returned.
	 * <br/>
	 * <br/>
	 * When a request is processed the resolved endpoint will return a representation
	 * of any type it chooses.
	 * If the request has the required representation class set then the
	 * response to the processed request
	 * will include a representation of this type or it will throw an exception.
	 * <br/>
	 * <br/>
	 * [[rsk 2009-03-24: I propose changing this to setRequiredRepresentationClass]]
	 *
	 * @param aRepresentationClass the required type of the returned resource
	 * representation. If null, the type is set to java.lang.Object, which
	 * indicates that any type may be returned.
	 */
	void setRepresentationClass(Class aRepresentationClass);
	
	/** Add a named argument to the request as an identifier.
	 * If the identifier is not absolute and a CWU a set on the context then this
	 * identifier is absolutized.
	 * @param aName the name of the argument
	 * @param aIdentifier the identifier of the argument
	 * @exception NKFException
	 */
	void addArgument(String aName, String aIdentifier) throws NKFException;
	
	/** Add a named argument to the request as an absolute string within the request identifier
	 * @param aName the name of the argument
	 * @param aAbsolute the string value of the argument
	 * @exception NKFException
	 */
	void addArgumentAsAbsolute(String aName, String aAbsolute) throws NKFException;
	
	/** Add the argument to a pass-by-value space which is injected into the request context.
	 *
	 * @param aName the name of the argument
	 * @param aRepresentation
	 */
	void addArgumentByValue(String aName, Object aRepresentation);
	
	/** Add a named pass-by-value argument to the request with the value and metadata
	 *  from a previous sub-request or internally constructed. This method is useful where
	 *  explicit metadata should be provided to the service for one of its arguments.
	 * @param aName the name of the argument
	 * @param aResponse the response from a previous request
	 */
	void addArgumentFromResponse(String aName, INKFResponseReadOnly aResponse);
	
	/** Add a single, name-less argument to the request.
	 * This is used for SINK and TRANSREPT requests to supply the representation to
	 * sink or transrept.
	 * @param aRepresentation the representation to sink to a resource or transrept to an alternate representation.
	 */
	void addPrimaryArgument(Object aRepresentation);
	
	/** Add a system argument (for use in transrept and sink
	 * requests with the value and metadata
	 *  from an internally constucted response. This method is useful where explicit metadata
	 *  should be provided
	 * @param aResponse
	 */	
	void addPrimaryArgumentFromResponse(INKFResponseReadOnly aResponse);
	
	/** Set the priority of the request.
	 * <br/><br/>
	 * [[rsk 2009-03-24: Change this to a request header?]]
	 * @param aPriority within the range of INKFRequestReadOnly.PRIORITY_MIN
	 * and INKFRequestReadOnly.PRIORITY_MAX
	 */
	void setPriority(int aPriority);
	
	/** Inject an additional non-durable space into the scope of the request to execute. This space
	 * will be innermost (i.e. resolved before anything)
	 * @param aSpace
	 */
	void injectRequestScope(ISpace aSpace);
	
	/** Inject an additional durable space into the scope of the request to execute. This space
	 * will be innermost (i.e. resolved before anything)
	 * @param aSpace
	 */
	void injectDurableRequestScope(ISpace aSpace);

	/** <b>Advanced usage only.</b> Set the scope under which the subrequest is
	 * resolved.
	 * @param aRequestScope the request scope stack structure 
	 */
	void setRequestScope(IRequestScopeLevel aRequestScope);
		
	/** Return the identifier for the request
	 * @throws NKFException
	 */
	String getIdentifier() throws NKFException;
	
	/** Set headers onto a request. Headers should not be misused, they are not
	 * arguments and should only be used for request process hinting.
	 * @param aKey
	 * @param aRepresentation
	 */
	void setHeader(String aKey, Object aRepresentation);
	
	/** Set headers onto a request. Headers should not be misused, they are not
	 * arguments and should only be used for request process hinting.
	 * @param aKey
	 * @param aRepresentation
	 * @param aSticky
	 */
	void setHeader(String aKey, Object aRepresentation, boolean aSticky);
}
