/******************************************************************************
 * (c) Copyright 2002,2003, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.util;

import java.text.DecimalFormat;
import java.text.NumberFormat;

/**
 * A low memory implementation of a multimap which preserves ordering. Implemented with an underlying array
 * and linear lookup time for get and remove.
 * @author  tab
 */
public final class OrderedCheapMultiStringMap
{
	/** the underlying list */
	private String[] mItems;
	/** used index */
	private int mIndex;
	
	public static final OrderedCheapMultiStringMap EMPTY = new OrderedCheapMultiStringMap(0);
	
	/** Creates a new instance of OrderedCheapMultiMap */
	public OrderedCheapMultiStringMap(int aSize)
	{	mItems = new String[aSize*2];
	}
	
	/** Clones a OrderedCheapMultiMap */
	public OrderedCheapMultiStringMap(OrderedCheapMultiStringMap aOther)
	{	mIndex= aOther.mIndex;
		mItems = new String[mIndex];
		System.arraycopy(aOther.mItems, 0, mItems, 0, mIndex);
	}
	
	public void restoreFrom(OrderedCheapMultiStringMap aOther)
	{	mIndex= aOther.mIndex;
		mItems = new String[mIndex];
		System.arraycopy(aOther.mItems, 0, mItems, 0, mIndex);
	}
	
	/** put a new key value pair in the collection */
	public void put(String aKey, String aValue)
	{	if (mIndex==mItems.length)
		{	String[] newItems = new String[mItems.length*2];
			System.arraycopy(mItems, 0,newItems, 0, mItems.length);
			mItems=newItems;
		}
		mItems[mIndex++]=aKey;
		mItems[mIndex++]=aValue;
	}
	
	/** return the number of pairs in the collection */
	public int size()
	{	return mIndex>>1;
	}
	
	/** return the key value at the given index */
	public String getKey(int aIndex)
	{	return mItems[aIndex+aIndex];
	}
	
	/** return the value value at the given index */
	public String getValue(int aIndex)
	{	return mItems[1+aIndex+aIndex];
	}
	
	/** returns first value matching given key */
	public String get(String aKey)
	{	String result = null;
		for (int i=0; i<mIndex; i+=2)
		{	String key=mItems[i];
			if (key.equals(aKey))
			{	result=mItems[i+1];
				break;
			}
		}
		return result;
	}
	/** removes first value matching given key and returns it */
	public String remove(String aKey)
	{	String result = null;
		for (int i=0; i<mIndex; i+=2)
		{	String key=mItems[i];
			if (key.equals(aKey))
			{	result=mItems[i+1];
				mIndex-=2;
				System.arraycopy(mItems, i+2, mItems, i, mIndex-i);
				break;
			}
		}
		return result;
	}
	
	public String[] asArray()
	{	String[] result = new String[mIndex];
		System.arraycopy(mItems, 0, result, 0, mIndex);
		return result;
	}
	
	public void sort()
	{	mergesort(0,size()-1);
	}
	
	/** In-place mergesort by key that preserves order of equal keys 
	 * @param lo
	 * @param hi
	 */
	private void mergesort(int lo, int hi)
	{	
		int mid = (lo + hi) / 2;

		if (lo<mid)
		{	mergesort(lo, mid);
			if (mid+1<hi)
			{	mergesort(mid + 1, hi);
			}
		}

		int endLo = mid;
		int startHi = mid + 1;
		while ((lo <= endLo) && (startHi <= hi))
		{	if (getKey(lo).compareTo(getKey(startHi))>0)
			{	int sh2=startHi+startHi;
				int lo2=lo+lo;
				String k=mItems[sh2];
				String v=mItems[sh2+1];
				System.arraycopy(mItems, lo2, mItems, lo2+2, sh2-lo2);
				mItems[lo2]=k;
				mItems[lo2+1]=v;
				
				endLo++;
				startHi++;
			}
			lo++;
		}
	}	
	
	public String toString()
	{	StringBuffer sb=new StringBuffer(256);
		sb.append('[');
		for (int i=0; i<mIndex; )
		{	if (i>0)
			{	sb.append(", ");
			}
			sb.append(mItems[i++]);
			sb.append("=\"");
			sb.append(mItems[i++]);
			sb.append('\"');
		}
		sb.append(']');
		return sb.toString();
	}
	
	
	public static void main(String[] a)
	{
		OrderedCheapMultiStringMap m = new OrderedCheapMultiStringMap(10);
		
		NumberFormat nf = new DecimalFormat("0000");
		for (int i=0; i<901; i++)
		{	
			int r = (int)(Math.random()*400.0);
			String rs = nf.format(r);
			m.put(rs,nf.format(i));
		}
		
		System.out.println(m);
		m.sort();
		
		for (int i=0; i<m.size()-1; i++)
		{	String k1=m.getKey(i);
			String k2=m.getKey(i+1);
			if (k1.compareTo(k2)>0)
			{	System.out.println("FAIL " +i+" "+k1+" "+k2);
			}
		}

		for (int i=0; i<m.size()-1; i++)
		{	String k1=m.getKey(i);
			String k2=m.getKey(i+1);
			if (k1.equals(k2) && m.getValue(i).compareTo(m.getValue(i+1))>0)
			{	System.out.println("ORDER BAD " +m.getValue(i)+" "+m.getValue(i+1));
			}
		}
		
		
		System.out.println(m);
		
	}
	
}