/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.util;
import java.util.*;
import java.util.regex.*;
import java.net.URL;

import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.layer0.util.DynamicURLClassLoader;
import org.netkernel.urii.ClassLoaderWithExports;
import org.netkernel.util.*;

/**
 * Classloader on a set of URLs and also a set of imported spaces.
 * Uses URLs in order, then imports, finally at the parent classloader.
 * @author  tab
 */
public class SpaceClassLoader extends DynamicURLClassLoader
{
	private ClassLoaderWithExports[] mImports;
	private Matcher[] mExports;
	private ClassLoader mParent;
	private boolean mInvalidated;
	private String mDebug;
	
	/**Creates a new instance of SpaceClassLoader
	 * @param aParent parent classloader
	 * @param aURLs the list of URLs to search for classes and resources
	 * @param aExports an optional list of export regex string matches, if null no classes are exported
	 * @param aDebug a debug string for toString() to be meaningful
	 */
	public SpaceClassLoader(ClassLoader aParent, List aURLs, List aExports, String aDebug)
	{	super(aURLs);
		mImports= new ClassLoaderWithExports[0];
		mParent = aParent;
		mDebug=aDebug;
		if (aExports!=null)
		{
			mExports = new Matcher[aExports.size()];
			int j=0;
			for (Iterator i=aExports.iterator(); i.hasNext(); )
			{	String pattern=(String)i.next();
				mExports[j++]=Pattern.compile(pattern).matcher("");
			}
		}
		else
		{	mExports=null;
		}
	}
	
	public ClassLoader getParentClassLoader()
	{	return mParent;
	}
	
	public void cleanup()
	{	mImports=null;
		mParent=null;
		super.cleanup();
	}
	
	public void invalidate()
	{	mInvalidated=true;
	}
	
	/** returns true after a classloader has been disposed */
	public boolean isInvalidated()
	{	boolean invalid=mInvalidated;
		if (!invalid)
		{
			for (int i=0; i<mImports.length; i++ )
			{	if (mImports[i].isInvalidated() && mImports[i].hasExports())
				{	invalid=true;
					break;
				}
			}
		}
		return invalid;
	}
	
	public boolean hasExports()
	{	return mExports!=null;
	}
	
	public boolean exports(String aName)
	{	boolean result;
		if (mExports==null)
		{	result=false;
		}
		else
		{	if (mExports.length==0)
			{	result=true;
			}
			else
			{	result = innerExports(aName);
			}
		}
		return result;
	}
	
	private synchronized boolean innerExports(String aName)
	{	boolean result=false;
		for (int i=0; !result && i<mExports.length; i++)
		{	mExports[i].reset(aName);
			result=mExports[i].matches();
		}
		return result;
	}
	
	
	/** dynamically add imports to the classloader */
	public void setImportedClassLoaders(ClassLoaderWithExports[] aClassLoaders)
	{	mImports=aClassLoaders;
	}
	
	public Class loadClass(String aName) throws ClassNotFoundException
	{	Class result;
		if (aName.startsWith("java."))
		{	result = findSystemClass(aName);
		}
		else
		{	result = innerLocalLoadClass(aName);
			if (result==null)
			{	result = loadImportedClass(aName);
			}
			if (result==null && mParent!=null)
			{	result = mParent.loadClass(aName);
			}
			if (result==null)
			{	throw new ClassNotFoundException(aName);
			}
		}
		return result;
	}
	
	public Class loadClassAvoidingParent(String aName) throws ClassNotFoundException
	{	Class result=null;
		if (aName.startsWith("java."))
		{	result = findSystemClass(aName);
		}
		if (result==null)
		{	result = innerLocalLoadClass(aName);
		}
		if (result==null)
		{	result = loadImportedClass(aName);
		}
		if (result==null)
		{	throw new ClassNotFoundException(aName);
		}
		return result;
	}
	
	private Class loadImportedClass(String aName)
	{	Class result = null;
		for (int i=0; i<mImports.length; i++ )
		{	if (mImports[i].exports(aName))
			{	try
				{	result = mImports[i].loadClass(aName);
					break;
				}
				catch (ClassNotFoundException e)
				{	/* the class wasn't found here */
				}
			}
		}
		return result;
	}	
	
	public URL getResource(String aName)
	{	URL result = super.getResource(aName);
		if (result==null)
		{	for (int i=0; i<mImports.length; i++ )
			{	if (mImports[i].exports(aName))
				{	result = mImports[i].getResource(aName);
					if (result!=null)
					{	break;
					}
				}
			}
		}
		if (result==null)
		{	result = mParent.getResource(aName);
		}
		return result;
	}	
	
	public String toString()
	{	return "SpaceClassLoader("+mDebug+")";
	}
	
	 public void setDebug(String aDebug)
     {        mDebug=aDebug;
     }
	
	/** append HDS representation of classloader structure
	 */
	 public void appendHDS(HDSBuilder b)
	 {
		 b.pushNode("SpaceClassLoader");
		 
		 b.pushNode("urls");
		 for (Iterator i=mURLs.iterator(); i.hasNext(); )
		 {	ClassPathElement cpe = (ClassPathElement)i.next();
		 	b.addNode("url", cpe.mBaseURL);
		 }
		 b.popNode();
		 
		 b.pushNode("imports");
		 for (ClassLoaderWithExports cl :mImports)
		 {	b.addNode("import",cl.toString());
		 }
		 b.popNode();
		 
		 for (ClassLoader parent=mParent; parent!=null; parent=parent.getParent())
		 {	b.pushNode("parent");
		 	b.addNode("toString", parent.toString());
			b.popNode();
		}
		 b.popNode();
	 }
}